/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.nachrichten.postfach;

import static java.util.Objects.*;

import java.util.Map;
import java.util.Optional;
import java.util.stream.Stream;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.context.annotation.Primary;
import org.springframework.stereotype.Service;

@Service
@Primary
public class PersistPostfachNachrichtServiceImpl implements PersistPostfachNachrichtService {

	private static final String NOT_IMPLEMENTED_MESSAGE = "Not implemented. Use gRPC API instead.";

	@Autowired
	@Qualifier("findVorgangRemoteService")
	private VorgangRemoteService vorgangRemoteService;

	@Autowired
	private AttachedItemRemoteService attachedItemRemoteService;

	@Autowired(required = false)
	@Qualifier("persistPostfachNachrichtByCommandService")
	private PersistPostfachNachrichtService vorgangManagerNachrichtService;

	@Override
	public void persistNachricht(Optional<String> userId, PostfachNachricht nachricht) {
		if (vorgangManagerNachrichtService != null) {
			vorgangManagerNachrichtService.persistNachricht(userId, nachricht);
			return;
		}
		throw new UnsupportedOperationException(NOT_IMPLEMENTED_MESSAGE);
	}

	@Override
	public Stream<Map<String, Object>> findByVorgangAsMap(String vorgangId) {
		if (vorgangManagerNachrichtService != null) {
			return vorgangManagerNachrichtService.findByVorgangAsMap(vorgangId);
		}
		throw new UnsupportedOperationException(NOT_IMPLEMENTED_MESSAGE);
	}

	@Override
	public Optional<Map<String, Object>> findById(String nachrichtId) {
		if (vorgangManagerNachrichtService != null) {
			return vorgangManagerNachrichtService.findById(nachrichtId);
		}
		throw new UnsupportedOperationException(NOT_IMPLEMENTED_MESSAGE);
	}

	@Override
	public void patch(String postfachMailId, Map<String, Object> propertyMap) {
		if (nonNull(vorgangManagerNachrichtService)) {
			vorgangManagerNachrichtService.patch(postfachMailId, propertyMap);
			return;
		}
		throw new UnsupportedOperationException(NOT_IMPLEMENTED_MESSAGE);
	}

	@Override
	public Map<String, Object> getById(String itemId) {
		if (vorgangManagerNachrichtService != null) {
			return vorgangManagerNachrichtService.getById(itemId);
		}
		throw new UnsupportedOperationException(NOT_IMPLEMENTED_MESSAGE);
	}

	@Override
	public String persistAttachment(String vorgangId, AttachmentFile attachment) {
		if (nonNull(vorgangManagerNachrichtService)) {
			return vorgangManagerNachrichtService.persistAttachment(vorgangId, attachment);
		}
		throw new UnsupportedOperationException(NOT_IMPLEMENTED_MESSAGE);
	}

	@Override
	public Stream<PostfachNachricht> findRueckfragen(String servicekontoType, String postfachId) {
		return vorgangRemoteService.findVorgangIds(servicekontoType, postfachId).flatMap(attachedItemRemoteService::findReplyAllowedPostfachNachrichts);
	}

	@Override
	public String persistAnswer(String rueckfrageId, PostfachNachricht answer) {
		if (nonNull(vorgangManagerNachrichtService)) {
			return vorgangManagerNachrichtService.persistAnswer(rueckfrageId, answer);
		}
		throw new UnsupportedOperationException(NOT_IMPLEMENTED_MESSAGE);
	}

	@Override
	public Stream<PostfachNachricht> findAnswers(String rueckfrageId) {
		if (nonNull(vorgangManagerNachrichtService)) {
			return vorgangManagerNachrichtService.findAnswers(rueckfrageId);
		}
		throw new UnsupportedOperationException(NOT_IMPLEMENTED_MESSAGE);
	}

}
