package de.ozgcloud.nachrichten.antragraum;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.UUID;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.opensaml.saml.saml2.core.Response;

import de.ozgcloud.nachrichten.NachrichtenManagerProperties;
import de.ozgcloud.nachrichten.postfach.PersistPostfachNachrichtService;
import de.ozgcloud.nachrichten.postfach.PostfachNachricht;
import de.ozgcloud.nachrichten.postfach.PostfachNachrichtTestFactory;

class AntragraumServiceTest {
	@Spy
	@InjectMocks
	private AntragraumService service;

	@Mock
	private AntragraumProperties properties;
	@Mock
	private NachrichtenManagerProperties nachrichtenManagerProperties;
	@Mock
	private PersistPostfachNachrichtService postfachNachrichtService;
	@Mock
	private Saml2Verifier verifier;
	@Mock
	private Saml2Parser parser;
	@Mock
	private Saml2Decrypter decrypter;

	@Nested
	class TestGetAntragsraumUrl {

		private static final String URL = "url";

		@Test
		void shouldReturnUrl() {
			when(properties.getUrl()).thenReturn(URL);

			var url = service.getAntragsraumUrl();

			assertThat(url).isEqualTo(URL);
		}

	}

	@Nested
	class TestFindRueckfragen {
		static final String SAML_TOKEN = "TOKEN";

		@Test
		void shouldCallVerify() {
			service.findRueckfragen(SAML_TOKEN);

			verify(service).verifyToken(anyString());
		}

		@Test
		void shouldCallVerifier() {
			service.findRueckfragen(SAML_TOKEN);

			verify(verifier).verify(anyString());
		}

		@Test
		void shouldCallDecrypt() {
			when(parser.parse(anyString())).thenReturn(mock(Response.class));

			service.findRueckfragen(SAML_TOKEN);

			verify(decrypter).decryptPostfachId(any(Response.class));
		}

		@Test
		void shouldCallPostfachService() {
			when(decrypter.decryptPostfachId(any())).thenReturn(GrpcFindRueckfrageRequestTestFactory.POSTFACH_ID);

			service.findRueckfragen(SAML_TOKEN);

			verify(postfachNachrichtService).findRueckfragen(anyString(), anyString());
		}
	}

	@Nested
	class TestSendRueckfragenAnswers {
		static final String SAML_TOKEN = "TOKEN";
		static final String RUECKFRAGE_ID = UUID.randomUUID().toString();
		static final PostfachNachricht NACHRICHT = PostfachNachrichtTestFactory.create();

		@Test
		void shouldCallVerify() {
			service.sendRueckfrageAnswer(SAML_TOKEN, RUECKFRAGE_ID, NACHRICHT);

			verify(service).verifyToken(anyString());
		}

		@Test
		void shouldCallVerifier() {
			service.findRueckfragen(SAML_TOKEN);

			verify(verifier).verify(anyString());
		}

		@Test
		void shouldCallPostfachService() {
			service.sendRueckfrageAnswer(SAML_TOKEN, RUECKFRAGE_ID, NACHRICHT);

			verify(postfachNachrichtService).persistAnswer(anyString(), any(PostfachNachricht.class));
		}
	}
}